%{
 /**********************************************************************
 @Lex-file{
    author              = "Daniel Trinkle",
    version             = "2.4",
    date                = "20 November 1996",
    time                = "16:02:48 MST",
    filename            = "detex.l",
    address             = "Department of Computer Science
                           Purdue University
                           1398 Computer Science Building
                           West Lafayette, IN 47907-1398
                           USA",
    telephone           = "+1 317 494-7844",
    FAX                 = "+1 317 494-0739",
    URL                 = "http://www.cs.purdue.edu/people/trinkle",
    checksum            = "28269 882 2865 22284",
    email               = "trinkle@cs.purdue.edu (Internet)",
    codetable           = "ISO/ASCII",
    keywords            = "detex, TeX, LaTeX",
    supported           = "yes",
    docstring           = "This is a utility for removing TeX and
                           LaTeX control sequences from a text stream.

			   The accompanying manual pages, which are
			   provided in several different formats,
			   provide complete documentation of the
			   program.

			   The checksum field above contains a CRC-16
                           checksum as the first value, followed by the
                           equivalent of the standard UNIX wc (word
                           count) utility output of lines, words, and
                           characters.  This is produced by Robert
                           Solovay's checksum utility.",
 }
 **********************************************************************/
#ifndef lint
static char	rcsid[] = "$Header: /u/sy/beebe/tex/detex/RCS/detex.l,v 1.1 1996/11/16 20:41:51 beebe Exp beebe $";
#endif

/*
 * detex [-?] [-a] [-c] [-e envlist] [-h] [-l] [-m]
 *	[-n] [-s] [-v] [-w] [file[.tex]]
 *
 *	This program is used to remove TeX or LaTeX constructs from a text
 *	file.
 *
 * Written by:
 *	Daniel Trinkle
 *	Department of Computer Science
 *	Purdue University
 *
 */

#include "config.h"
#include "detex.h"

#if defined(HAVE_STDIO_H)
#include <stdio.h>
#endif

#if defined(HAVE_CTYPE_H)
#include <ctype.h>
#endif

#if defined(HAVE_STRING_H)
#include <string.h>
#endif

#if defined(HAVE_SYS_PARAM_H)
#include <sys/param.h>	/* for NOFILE and MAXPATHLEN */
#else
#if !defined(NOFILE)
#define NOFILE		10	/* on IBM PC DOS, this depends on
				FILES=nnn setting in config.sys file */
#endif
#if !defined(MAXPATHLEN)
#define MAXPATHLEN	1024	/* big enough for most applications */
#endif
#endif

#if defined(HAVE_STDLIB_H)
#include <stdlib.h>
#endif

#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif

int		main ARGS((int cArgs, char *rgsbArgs[]));
extern int	yylex ARGS((void));

#if defined(__cplusplus) && defined(YYWRAP_HAS_C_LINKAGE)
extern "C" {
#endif
#define YY_SKIP_YYWRAP	1 /* to prevent flex declaring its own, possibly incompatible, yywrap() */
int		yywrap ARGS((void));
#if defined(__cplusplus) && defined(YYWRAP_HAS_C_LINKAGE)
}
#endif

static void     AddInclude ARGS((const char *sbFile));
static void	Author ARGS((void));
static int      BeginEnv ARGS((const char *sbEnv));
static int      EndEnv ARGS((const char *sbEnv));
static void	ErrorExit ARGS((const char *sb1));
static void	Help ARGS((void));
static void     IncludeFile ARGS((const char *sbFile));
static int      InList ARGS((const char *sbFile));
static void     InputFile ARGS((const char *sbFile));
static void     MarkOmitted ARGS((const char *s));
static void	PrintLines ARGS((FILE *fpout, const char *lines[]));
static int      SeparateList ARGS((char *sbList, const char *rgsbList[], int chSep, int csbMax));
static void     SetEnvIgnore ARGS((char *sbEnvList));
static void     SetInputPaths ARGS((void));
static char	*Strdup ARGS((const char *s));
static FILE *   TexOpen ARGS((const char *sbFile));
static void	Version ARGS((void));
static void     Warning ARGS((const char *sb1, const char *sb2));

#define	CITEBEGIN	if (!fLatex || !fCite) BEGIN
#define	IGNORE		if (fSpace && !fWord) putchar(' ')
#define	LaBEGIN		if (fLatex) BEGIN
#define	NEWLINE		if (!fWord) putchar('\n')
#define	SPACE		if (!fWord) putchar(' ')
#define EMDASH		if (!fWord) fputs(" --- ",stdout)
#define ENDASH		if (!fWord) fputs(" -- ",stdout)
#define SLASH		if (!fWord) fputs(" / ",stdout)

const char	*rgsbEnvIgnore[MAXENVS];	/* list of environments ignored */
char		*rgsbIncList[MAXINCLIST];	/* list of includeonly files */
const char	*rgsbInputPaths[MAXINPUTPATHS];	/* list of input paths in order */
char		sbCurrentEnv[CCHMAXENV];	/* current environment being ignored */
const char	*sbProgName;			/* name we were invoked with */
FILE	*rgfp[NOFILE+1];		/* stack of input/include files */
int	cfp = 0;			/* count of files in stack */
int	csbEnvIgnore;			/* count of environments ignored */
int	csbIncList = 0;			/* count of includeonly files */
int	csbInputPaths;			/* count of input paths */
int	fLatex = 0;			/* flag to indicated delatex */
int	fMarkomitted = 0;		/* flag to request marking of omitted */
int	fWord = 0;			/* flag for -w option */
int	fFollow = 1;			/* flag to follow input/include */
int	fCite = 0;			/* flag to echo \cite and \ref args */
int	fSpace = 0;			/* flag to replace \cs with space */
%}

S	[ \t\n]*
W	[a-zA-Z]+

%Start Def Disp IncOnly Input Math Nrm Ctl
%Start LaBegin LaDisp LaEnd LaEnv LaForm LaInc LaMacro LaVerb

%%
<Nrm>"%"{S}-[*]-LaTeX-[*]-.*			{fLatex = 1;}

<Nrm>"%"{S}-[*]-TeX-[*]-.*			{fLatex = 0;}

<Nrm>"%".*		/* ignore comments */	;

<Nrm>"\\begin"{S}"{"{S}"document"{S}"}"		{fLatex = 1; IGNORE;}

<Nrm>"\\begin"     /* environment start */	{LaBEGIN LaBegin; IGNORE;}

<LaBegin>{S}"{"{S}"verbatim"{S}"}"		{   if (BeginEnv("verbatim"))
							BEGIN LaEnv;
						    else
							BEGIN LaVerb;
						    IGNORE;
						}

<LaVerb>"\\end"{S}"{"{S}"verbatim"{S}"}" /* verbatim mode */	{BEGIN Nrm; IGNORE;}
<LaVerb>.					ECHO;

<LaBegin>{W}					{   if (BeginEnv((char*)&yytext[0]))
							BEGIN LaEnv;
						    else
							BEGIN LaMacro;
						    IGNORE;
						}
<LaBegin>"\n"					NEWLINE;
<LaBegin>.					;

<LaEnv>"\\end" /* absorb some environments */	{LaBEGIN LaEnd; IGNORE;}
<LaEnv>"\n"					NEWLINE;
<LaEnv>.					;

<LaEnd>{W}		/* end environment */	{   if (EndEnv((char*)&yytext[0]))
							BEGIN Nrm;
						    IGNORE;
						}
<LaEnd>"}"					{BEGIN LaEnv; IGNORE;}
<LaEnd>"\n"					NEWLINE;
<LaEnd>.					;

<Nrm>"\\altcite"				{CITEBEGIN LaMacro; MarkOmitted(MARKCITE); IGNORE;}
<Nrm>"\\bibitem"	/* ignore args  */	{LaBEGIN LaMacro; IGNORE;}
<Nrm>"\\bibliography"	/* of these \cs */	{LaBEGIN LaMacro; IGNORE;}
<Nrm>"\\bibstyle"				{LaBEGIN LaMacro; IGNORE;}
<Nrm>"\\cite"					{CITEBEGIN LaMacro; MarkOmitted(MARKCITE); IGNORE;}
<Nrm>"\\citeA"					{CITEBEGIN LaMacro; MarkOmitted(MARKCITE); IGNORE;}
<Nrm>"\\citeANP"				{CITEBEGIN LaMacro; MarkOmitted(MARKCITE); IGNORE;}
<Nrm>"\\citeN"					{CITEBEGIN LaMacro; MarkOmitted(MARKCITE); IGNORE;}
<Nrm>"\\citeNP"					{CITEBEGIN LaMacro; MarkOmitted(MARKCITE); IGNORE;}
<Nrm>"\\citeyear"				{CITEBEGIN LaMacro; MarkOmitted(MARKCITE); IGNORE;}
<Nrm>"\\citeyearNP"				{CITEBEGIN LaMacro; MarkOmitted(MARKCITE); IGNORE;}
<Nrm>"\\documentstyle"				{LaBEGIN LaMacro; IGNORE;}
<Nrm>"\\end"					{LaBEGIN LaMacro; IGNORE;}
<Nrm>"\\fullcite"				{CITEBEGIN LaMacro; MarkOmitted(MARKCITE); IGNORE;}
<Nrm>"\\fullciteA"				{CITEBEGIN LaMacro; MarkOmitted(MARKCITE); IGNORE;}
<Nrm>"\\glossary"				{LaBEGIN LaMacro; MarkOmitted(MARKGLOSSARY); SPACE;}
<Nrm>"\\index"					{LaBEGIN LaMacro; MarkOmitted(MARKINDEX); SPACE;}
<Nrm>"\\label"					{LaBEGIN LaMacro; MarkOmitted(MARKLABEL); IGNORE;}
<Nrm>"\\pageref"				{CITEBEGIN LaMacro; MarkOmitted(MARKPAGEREF); IGNORE;}
<Nrm>"\\ref"					{CITEBEGIN LaMacro; MarkOmitted(MARKREF); IGNORE;}
<Nrm>"\\shortcite"				{CITEBEGIN LaMacro; MarkOmitted(MARKCITE); IGNORE;}
<Nrm>"\\shortciteA"				{CITEBEGIN LaMacro; MarkOmitted(MARKCITE); IGNORE;}
<LaMacro>"}"					BEGIN Nrm;
<LaMacro>"\n"					NEWLINE;
<LaMacro>.					;

<Nrm>"\\def"		/* ignore def begin */	{BEGIN Def; IGNORE;}
<Def>"{"					BEGIN Nrm;
<Def>"\n"					NEWLINE;
<Def>.						;

<Nrm>"\\("		/* formula mode */	{LaBEGIN LaForm; MarkOmitted(MARKMATHMODE); IGNORE;}
<LaForm>"\\)"					BEGIN Nrm;
<LaForm>"\n"					NEWLINE;
<LaForm>.					;

<Nrm>"\\["		/* display mode */	{LaBEGIN LaDisp; MarkOmitted(MARKMATHMODE); IGNORE;}
<LaDisp>"\\]"					BEGIN Nrm;
<LaDisp>"\n"					NEWLINE;
<LaDisp>.					;

<Nrm>"$$"		/* display mode */	{BEGIN Disp; MarkOmitted(MARKMATHMODE); IGNORE;}
<Disp>"$$"					BEGIN Nrm;
<Disp>"\n"					NEWLINE;
<Disp>.						;

<Nrm>"$"		/* math mode */		{BEGIN Math; MarkOmitted(MARKMATHMODE); IGNORE;}
<Math>"$"					BEGIN Nrm;
<Math>"\n"					NEWLINE;
<Math>"\\$"					;
<Math>.						;

<Nrm>"\\include"	/* process files */	{LaBEGIN LaInc; IGNORE;}
<LaInc>[^{ \t\n}]+				{   IncludeFile((char*)&yytext[0]);
						    BEGIN Nrm;
						}
<LaInc>"\n"					NEWLINE;
<LaInc>.					;

<Nrm>"\\includeonly"				{BEGIN IncOnly; IGNORE;}
<IncOnly>[^{ \t,\n}]+				AddInclude((char*)&yytext[0]);
<IncOnly>"}"					{   if (csbIncList == 0)
							rgsbIncList[csbIncList++] = NULL;
						    BEGIN Nrm;
						}
<IncOnly>"\n"					NEWLINE;
<IncOnly>.					;

<Nrm>"\\input"					{BEGIN Input; IGNORE;}
<Input>[^{ \t\n}]+				{   InputFile((char*)&yytext[0]);
						    BEGIN Nrm;
						}
<Input>"\n"					NEWLINE;
<Input>.					;

<Nrm>"\\slash"					SLASH;
<Nrm>"\\endash"					ENDASH;
<Nrm>"\\emdash"					EMDASH;

<Nrm>\\[a-zA-Z@]+	/* ignore other \cs */	{BEGIN Ctl; IGNORE;}
<Nrm>"\\ "					SPACE;
<Nrm>\\.					IGNORE;
<Ctl>\\[a-zA-Z@]+				IGNORE;
<Ctl>[a-zA-Z@0-9]*[-'=`][^ \t\n{]*		IGNORE;
<Ctl>"\n"					{BEGIN Nrm; NEWLINE;}
<Ctl>[ \t{]*					{BEGIN Nrm; IGNORE;}
<Ctl>.						{yyless(0);BEGIN Nrm;}

<Nrm>[{}\\|]	/* special characters */	IGNORE;
<Nrm>[!?]"`"					IGNORE;
<Nrm>~						SPACE;

<Nrm>{W}[']*{W}					{   if (fWord)
							(void)printf("%s\n", yytext);
						    else
							ECHO;
						}
<Nrm>[0-9]+					if (!fWord) ECHO;
<Nrm>(.|\n)					if (!fWord) ECHO;
%%
/******
** main --
**	Set sbProgName to the base of arg 0.
**	Set the input paths.
**	Check for options
**		-c		echo LaTeX \cite, \ref, and \pageref values
**		-e <env-list>	list of LaTeX environments to ignore
**		-l		force latex mode
**		-m		mark math modes
**		-n		do not follow \input and \include
**		-w		word only output
**	Set the list of LaTeX environments to ignore.
**	Process each input file.
**	If no input files are specified on the command line, process stdin.
******/

#if STDC
int
main(int cArgs, char *rgsbArgs[])
#else
int
main(cArgs,rgsbArgs)
int	cArgs;
char	*rgsbArgs[];
#endif
{
	char	*pch;
	char *sbEnvList;
	char sbBadOpt[2];
	int	fSawFile = 0, iArgs = 1;

	sbEnvList = Strdup(DEFAULTENV);

	/* get base name and decide what we are doing, detex or delatex */
	if ((sbProgName = strrchr(rgsbArgs[0], '/')) != NULL)
	    sbProgName++;
	else
	    sbProgName = rgsbArgs[0];

	/* set rgsbInputPaths for use with TexOpen() */
	SetInputPaths();

	/* process command line options */
	while (iArgs < cArgs && *(pch = rgsbArgs[iArgs]) == CHOPT) {
		if (*(pch+1) == CHOPT)	/* allow GNU/POSIX-style --options */
			++pch;
		while (*++pch)
		    switch (isupper(*pch) ? tolower(*pch) : *pch) {
		    case CHAUTHOR:
			Author();	/* never returns */
			break;
		    case CHCITEOPT:
			fCite = 1;
			break;
		    case CHENVOPT:
			sbEnvList = rgsbArgs[++iArgs];
			break;
		    case CHHELP:
		    case CHQUERY:
		        Help();		/* never returns */
			break;
		    case CHLATEXOPT:
			fLatex = 1;
			break;
		    case CHMARKOMITTED:
			fMarkomitted = 1;
			break;
		    case CHNOFOLLOWOPT:
			fFollow = 0;
			break;
		    case CHSPACEOPT:
			fSpace = 1;
			break;
		    case CHVERSION:
			Version();	/* never returns */
			break;
		    case CHWORDOPT:
			fWord = 1;
			break;
		    default:
			sbBadOpt[0] = *pch;
			sbBadOpt[1] = '\0';
			Warning("unknown option ignored -", sbBadOpt);
		    }
		iArgs++;
	}
	SetEnvIgnore(sbEnvList);

	/* process input files */
	for (; iArgs < cArgs; iArgs++) {
	    fSawFile++;
	    if ((yyin = TexOpen(rgsbArgs[iArgs])) == NULL) {
		Warning("can't open file", rgsbArgs[iArgs]);
		continue;;
	    }
	    BEGIN Nrm;
	    (void)yylex();
	}

	/* if there were no input files, assume stdin */
	if (!fSawFile) {
	    yyin = stdin;
	    BEGIN Nrm;
	    (void)yylex();
	}
	if (YYSTATE != Nrm)
	    ErrorExit("input contains an unterminated mode or environment");
	return(0);
}

/******
** yywrap -- handles EOF for lex.  Check to see if the stack of open files
**	has anything on it.  If it does, set yyin to the to value.  If not
**	return the termination signal for lex.
******/

int
yywrap(VOID_ARG)
{
	(void)fclose(yyin);
	if (cfp > 0) {
	    yyin = rgfp[--cfp];
	    return(0);
	}
	return(1);
}

/******
** BeginEnv -- checks to see if sbEnv is in the list rgsbEnvIgnore.  If it
**	is, sbCurrentEnv is set to sbEnv.
******/

#if STDC
static int
BeginEnv(const char *sbEnv)
#else
static int
BeginEnv(sbEnv)
const char	*sbEnv;
#endif
{
	int	i;

	if (!fLatex) return(0);
	for (i = 0; i < csbEnvIgnore; i++)
	    if (strcmp(sbEnv, rgsbEnvIgnore[i]) == 0) {
		(void)strncpy(sbCurrentEnv, sbEnv, CCHMAXENV);
		return(1);
	    }
	return(0);
}

/******
** EndEnv -- checks to see if sbEnv is the current environment being ignored.
******/

#if STDC
static int
EndEnv(const char *sbEnv)
#else
static int
EndEnv(sbEnv)
const char	*sbEnv;
#endif
{
	if (!fLatex) return(0);
	if (strcmp(sbEnv, sbCurrentEnv) == 0)
	    return(1);
	return(0);
}

/******
** ErrorExit -- print an error message preceded by the program name.
**	Stdout is flushed and detex exits.
******/

#if STDC
static void
ErrorExit(const char *sb1)
#else
static void
ErrorExit(sb1)
const char	*sb1;
#endif
{
	(void)fflush(stdout);
	(void)fprintf(stderr, "%s: error: %s\n", sbProgName, sb1);
	exit(EXIT_FAILURE);
}

/******
** Help -- print a brief help message on stderr and exit with a success
**	status code.
******/

static void
Help(VOID_ARG)
{
	static const char *lines[] =
	{
		"Usage:\tdetex [-?] [-a] [-c] [-e envlist] [-h] [-l] [-m] [-n]",
		"\t[-s] [-v] [-w] [filename[.tex] ...]",
		"",
		"Options:",
		"\t-?\t\tdisplay this help text",
		"\t-a\t\tdisplay author credit and exit",
		"\t-c\t\techo citation and cross-reference arguments",
		"\t-e envlist\treplace default ignored environments",
		"\t-h\t\tdisplay this help text",
		"\t-l\t\tforce LaTeX mode",
		"\t-n\t\tignore \\input and \\include commands",
		"\t-s\t\told-style control-sequence stripping",
		"\t-v\t\tdisplay version number and exit",
		"\t-w\t\toutput one word per line",
		(const char*)NULL
	};

	PrintLines(stderr,lines);
	exit(EXIT_SUCCESS);
}


/******
** InputFile -- push the current yyin and open sbFile.  If the open fails,
**	the sbFile is ignored.
******/

#if STDC
static void
InputFile(const char *sbFile)
#else
static void
InputFile(sbFile)
const char	*sbFile;
#endif
{
	if (!fFollow)
	    return;
	rgfp[cfp++] = yyin;
	if ((yyin = TexOpen(sbFile)) == NULL) {
	    Warning("can't open \\input file", sbFile);
	    yyin = rgfp[--cfp];
	}
}

/******
** IncludeFile -- if sbFile is not in the rgsbIncList, push current yyin
**	and open sbFile.  If the open fails, the sbFile is ignored.
******/

#if STDC
static void
IncludeFile(const char *sbFile)
#else
static void
IncludeFile(sbFile)
const char	*sbFile;
#endif
{
	if (!fFollow)
	    return;
	if (!InList(sbFile))
	    return;
	rgfp[cfp++] = yyin;
	if ((yyin = TexOpen(sbFile)) == NULL) {
	    Warning("can't open \\include file", sbFile);
	    yyin = rgfp[--cfp];
	}
}

/******
** AddInclude -- adds sbFile to the rgsbIncList and increments csbIncList.
**	If the include list is too long, sbFile is ignored.
******/

#if STDC
static void
AddInclude(const char *sbFile)
#else
static void
AddInclude(sbFile)
const char	*sbFile;
#endif
{
	if (!fFollow)
	    return;
	if (csbIncList >= MAXINCLIST)
	    Warning("\\includeonly list is too long, ignoring", sbFile);
	rgsbIncList[csbIncList++] = Strdup(sbFile);
}

/******
** Author -- print an author credit on stderr and exit with a success
**	status code
******/

static void
Author(VOID_ARG)
{
	static const char *lines[] =
	{
		"Author:",
		"\tDaniel Trinkle",
		"\tDepartment of Computer Science",
		"\tPurdue University",
		"\t1398 Computer Science Building",
		"\tWest Lafayette, IN 47907-1398",
		"\tUSA\n",
		"\tEmail: trinkle@cs.purdue.edu",
		"\tWWW URL: http://www.cs.purdue.edu/people/trinkle",
		(const char*)NULL
	};

	PrintLines(stderr,lines);
	exit(EXIT_SUCCESS);
}

/******
** InList -- checks to see if sbFile is in the rgsbIncList.  If there is
**	no list, all files are assumed to be "in the list".
******/

#if STDC
static int
InList(const char *sbFile)
#else
static int
InList(sbFile)
const char	*sbFile;
#endif
{
	char	*pch, sbBase[MAXPATHLEN];
	int	i;

	if (csbIncList == 0)	/* no list */
	    return(1);
	(void)strcpy(sbBase, sbFile);
	if ((pch = strrchr(sbBase, '.')) != NULL)
	    *pch = '\0';
	i = 0;
	while ((i < csbIncList) && rgsbIncList[i])
	    if (strcmp(rgsbIncList[i++], sbBase) == 0)
	        return(1);
	return(0);
}

/******
** MarkOmitted -- outputs special marker to show location of math mode
******/

#if STDC
static void
MarkOmitted(const char *s)
#else
static void
MarkOmitted(s)
const char *s;
#endif
{
    if (fMarkomitted)
	fputs(s,stdout);
}

/******
** PrintLines() -- print a NULL-terminated array of lines
******/

#if STDC
static void
PrintLines(FILE *fpout, const char *lines[])
#else
static void
PrintLines(fpout,lines)
FILE *fpout;
const char *lines[];
#endif
{
	int k;

	for (k = 0; lines[k] != (const char*)NULL; ++k)
		(void)fprintf(fpout,"%s\n", lines[k]);
}


/******
** SetEnvIgnore -- sets rgsbEnvIgnore to the values indicated by the
**	sbEnvList.
******/

#if STDC
static void
SetEnvIgnore(char *sbEnvList)
#else
static void
SetEnvIgnore(sbEnvList)
char	*sbEnvList;
#endif
{
	csbEnvIgnore = SeparateList(sbEnvList, rgsbEnvIgnore, CHENVSEP, MAXENVS);
	if (csbEnvIgnore == ERROR)
	    ErrorExit("The environment list contains too many environments");
}

/******
** SetInputPaths -- sets rgsbInputPaths to the values indicated by the
**	TEXINPUTS environment variable if set or else DEFAULTINPUTS.
******/

static void
SetInputPaths(VOID_ARG)
{
    const char *sb;
    char *sbPaths;

    if ((sb = getenv("TEXINPUTS")) == NULL)
	sbPaths = Strdup(DEFAULTINPUTS);
    else
	sbPaths = Strdup(sb);

    csbInputPaths = SeparateList(sbPaths, rgsbInputPaths, CHPATHSEP, MAXINPUTPATHS);
    if (csbInputPaths == ERROR)
	ErrorExit("TEXINPUTS environment variable has too many paths");
}

/******
** SeparateList -- takes a chSep separated list sbList, replaces the
**	chSep's with NULLs and sets rgsbList[i] to the beginning of
**	the ith word in sbList.  The number of words is returned.  A
**	ERROR is returned if there are more than csbMax words.
******/

#if STDC
static int
SeparateList(char *sbList, const char *rgsbList[], int chSep, int csbMax)
#else
static int
SeparateList(sbList, rgsbList, chSep, csbMax)
char	*sbList;
const char *rgsbList[];
int	chSep;
int	csbMax;
#endif
{
	int	csbList = 0;

	while (sbList && *sbList && csbList < csbMax) {
	    rgsbList[csbList++] = sbList;
	    if ((sbList = strchr(sbList, chSep), sbList))
		*sbList++ = '\0';
	}
	return((sbList && *sbList) ? ERROR : csbList);
}

/******
** Strdup -- allocate a copy of a string
******/

#if STDC
static char *
Strdup(const char *s)
#else
static char *
Strdup(s)
const char *s;
#endif
{
    char *p;
    p = (char*)malloc(strlen(s) + 1);
    if (p == (char*)NULL)
	ErrorExit("Out of dynamic memory");
    return (strcpy(p,s));
}


/******
** TexOpen -- tries to open sbFile in each of the rgsbInputPaths in turn.
**	For each input path the following order is used:
**		file.tex - must be as named, if not there go to the next path
**		file.ext - random extension, try it
**		file     - base name, add .tex and try it
**		file     - try it as is
**	Notice that if file exists in the first path and file.tex exists in
**	one of the other paths, file in the first path is what is opened.
**	If the sbFile begins with a '/', no paths are searched.
******/

#if STDC
static FILE *
TexOpen(const char *sbFile)
#else
static FILE *
TexOpen(sbFile)
const char	*sbFile;
#endif
{
	char	*pch, *sbNew;
	FILE	*fp;
	int	iPath;
	static char	sbFullPath[MAXPATHLEN];

	for (iPath = 0; iPath < csbInputPaths; iPath++) {
	    if (*sbFile == '/') {	/* absolute path */
		(void)sprintf(sbFullPath, "%s", sbFile);
		iPath = csbInputPaths;	/* only check once */
	    } else
		(void)sprintf(sbFullPath, "%s/%s", rgsbInputPaths[iPath], sbFile);

	    /* If sbFile ends in .tex then it must be there */
	    if ((pch = strrchr(sbFullPath, '.')) != NULL
			&& (strcmp(pch, ".tex") == 0))
		if ((fp = fopen(sbFullPath, "r")) != NULL)
		    return(fp);
		else
		    continue;

	    /* If sbFile ends in .ltx then it must be there */
	    if ((pch = strrchr(sbFullPath, '.')) != NULL
			&& (strcmp(pch, ".ltx") == 0))
		if ((fp = fopen(sbFullPath, "r")) != NULL)
		{
		    fLatex = 1;
		    return(fp);
		}
		else
		    continue;

	    /* if .<ext> then try to open it.  the '.' represents   */
	    /* the beginning of an extension if it is not the first */
	    /* character and it does not follow a '.' or a '/'      */
	    if (pch != NULL && pch > &(sbFullPath[0])
                    && *(pch - 1) != '.' && *(pch - 1) != '/'
		    && (fp = fopen(sbFullPath, "r")) != NULL)
		return(fp);

	    /* just base name, add .tex to the name */
	    sbNew = (char*)malloc((unsigned)(strlen(sbFullPath) + 5));
	    (void)strcpy(sbNew, sbFullPath); /* b */
	    (void)strcat(sbNew, ".tex");
	    if ((fp = fopen(sbNew, "r")) != NULL)
		return(fp);

	    /* just base name, add .ltx to the name */
	    (void)strcpy(sbNew, sbFullPath); /* b */
	    (void)strcat(sbNew, ".ltx");
	    if ((fp = fopen(sbNew, "r")) != NULL)
	    {
		fLatex = 1;
		return(fp);
	    }

	    /* try sbFile regardless */
	    if ((fp = fopen(sbFullPath, "r")) != NULL)
		return(fp);
	}
	return((FILE *)NULL);
}

/******
** Version -- print a version number on stderr and exit with a success
**	status code
******/

static void
Version(VOID_ARG)
{
	static const char *lines[] =
	{
		"This is detex, Version 2.4, 17 November 1996",
		(const char*)NULL
	};

	PrintLines(stderr,lines);
	exit(EXIT_SUCCESS);
}


/******
** Warning -- print a warning message preceded by the program name.
******/

#if STDC
static void
Warning(const char *sb1, const char *sb2)
#else
static void
Warning(sb1, sb2)
const char	*sb1, *sb2;
#endif
{
	(void)fprintf(stderr, "%s: warning: %s %s\n", sbProgName, sb1, sb2);
}
