.\" -*-nroff-*-
.\" ====================================================================
.\"  @Troff-man-file{
.\"     author          = "Daniel Trinkle",
.\"     version         = "2.4",
.\"     date            = "17 November 1996",
.\"     time            = "09:36:20 MST",
.\"     filename        = "detex.man",
.\"     address         = "Department of Computer Science
.\"                        Purdue University
.\"                        1398 Computer Science Building
.\"                        West Lafayette, IN 47907-1398
.\"                        USA",
.\"     telephone       = "+1 317 494-7844",
.\"     FAX             = "+1 317 494-0739",
.\"     URL             = "http://www.cs.purdue.edu/people/trinkle",
.\"     checksum        = "23298 343 1317 9631",
.\"     email           = "trinkle@cs.purdue.edu (Internet)",
.\"     codetable       = "ISO/ASCII",
.\"     keywords        = "detex, TeX, LaTeX",
.\"     supported       = "yes",
.\"     docstring       = "This is the UNIX manual page documentation
.\"                        for detex, a utility for removing TeX and
.\"                        LaTeX control sequences from a text stream.
.\"
.\"                        The checksum field above contains a CRC-16
.\"                        checksum as the first value, followed by the
.\"                        equivalent of the standard UNIX wc (word
.\"                        count) utility output of lines, words, and
.\"                        characters.  This is produced by Robert
.\"                        Solovay's checksum utility.",
.\"  }
.\" ====================================================================
.if t .ds La L\\h'-0.24m'\\v'-0.15v'\\s-2A\\s+2\\h'-0.15m'\\v'0.15v'T\\h'-0.1667m'\\v'0.20v'E\\v'-0.20v'\\h'-0.125m'X
.if n .ds La LaTeX
.if t .ds Te T\\h'-0.1667m'\\v'0.20v'E\\v'-0.20v'\\h'-0.125m'X
.if n .ds Te TeX
.\"======================================================================
.TH DETEX 1 "17 November 1996" "Version 2.4"
.\"======================================================================
.SH NAME
detex \- a filter to strip TeX commands from a .tex file.
.\"======================================================================
.SH SYNOPSIS
.B detex
.RB [ \-? ]
.RB [ \-a ]
.RB [ \-c ]
.RB [ \-e
.IR "envlist" ]
.RB [ \-h ]
.RB [ \-l ]
.RB [ \-m ]
.RB [ \-n ]
.RB [ \-s ]
.RB [ \-v ]
.RB [ \-w ]
.RI [ filename [.tex]
.IR .\|.\|. ]
.\"======================================================================
.SH DESCRIPTION
.B detex
reads each file in sequence, and removes all comments, all
.I \*(Te\&
control sequences, and all text in inline math mode and display math
mode, and writes the remainder on the standard output.
.PP
By default,
.B detex
follows
.I \einput
commands.  If a file cannot be opened, a warning
message is printed and the command is ignored.
.PP
If no input files are given on the command line,
.B detex
reads from standard input.
.PP
.B detex
assumes the standard character classes (category codes) are being used
for
.IR \*(Te\& ,
and it allows white space between control sequences and magic
characters like `{' when recognizing things like
.I \*(La\&
environments.
.PP
The
.B TEXINPUTS
environment variable is used to find
.I \einput
and
.I \einclude
files.
.\"======================================================================
.SH OPTIONS
Command-line options are single letters, and letter case is ignored.
For compatibility with GNU and POSIX conventions, options may be
introduced by either a single or a double hyphen:
.B \-v
and
.B \--v
are equivalent.
.PP
Multiple single-letter options can be collapsed into a single
multiletter option:
.B "\-a \-c \-l"
and
.B "\-acl"
are equivalent.
.PP
To avoid confusion with options, if a filename
begins with a hyphen, it must be disguised by a
leading absolute or relative directory path, e.g.
.I /tmp/-foo.tex
or
.IR ./-foo.tex .
.\"-----------------------------------------------
.TP \w'e_envlist'u+3n
.B \-?
Display a brief help message on
.IR stderr ,
and then exit immediately with a success status code (0 on UNIX).
.\"-----------------------------------------------
.TP
.B \-a
Display an author credit on
.IR stderr ,
and then exit immediately with a success status code (0 on UNIX).
.\"-----------------------------------------------
.TP
.B \-c
In \*(La\& mode, echo the arguments to
.IR \ecite ,
.IR \eref ,
and
.IR \epageref
macros; they are otherwise normally discarded.  This option can be
useful when sending the output to a style checker.
.IP
Besides
.IR \ecite ,
.B detex
also recognizes the
.IR authordate1-4 ,
.IR chicago ,
and
.I harvard
citation command variants:
.IR \ealtcite ,
.IR \eciteA ,
.IR \eciteANP ,
.IR \eciteN ,
.IR \eciteNP ,
.IR \eciteyear ,
.IR \eciteyearNP ,
.IR \efullcite ,
.IR \efullciteA ,
.IR \epageref ,
.IR \eref ,
.IR \eshortcite ,
and
.IR \eshortciteA .
.\"-----------------------------------------------
.TP
.BI \-e " envlist"
Text in various environments of
.I \*(La\&
is ignored.  The default ignored environments are
.IR align ,
.IR alignat ,
.IR array ,
.IR eqnarray ,
.IR equation ,
.IR figure ,
.IR gather ,
.IR multline ,
.IR picture ,
.IR table
and
.IR verbatim .
The
.B \-e
option can be used to specify a comma-separated
.I list
of environments to ignore.  The list replaces the defaults, so
specifying an empty list effectively prevents all environments from
being ignored.
.\"-----------------------------------------------
.TP
.B \-h
Display a brief help message on
.IR stderr ,
and then exit immediately with a success status code (0 on UNIX).
.\"-----------------------------------------------
.TP
.B \-l
.B detex
normally assumes that it is dealing with plain \*(Te\&, or a variant
such as extended plain \*(Te\&, or AmS\*(Te\&.  However, if the magic
sequence
.I \ebegin{document}
appears in the text, or an Emacs-style mode comment
.nf
.I "% -*-LaTeX-*-"
.fi
is found, or the input file has a
.I .ltx
extension, then
.B detex
assumes it is dealing with \*(La\& source and it recognizes additional
constructs used in
.IR \*(La\& .
These include the
.I \einclude
and
.I \eincludeonly
commands.
The
.B \-l
option can be used to force \*(La\& mode, which is useful if the input
files would not otherwise be recognized as \*(La\& files.
.IP
An Emacs-style mode comment
.nf
.I "% -*-TeX-*-"
.fi
turns off \*(La\& mode.
.\"-----------------------------------------------
.TP
.B \-m
Instead of completely discarding math mode, citation, and
cross-reference text, mark their presence by a single word
.IR [CITE] ,
.IR [LABEL] ,
.IR [MATH] ,
.IR [PAGEREF] ,
or
.IR [REF] .
This is useful when the output is filtered by the doubled-word
utility,
.BR dw (1),
because it reduces the number of bogus warnings.
.\"-----------------------------------------------
.TP
.B \-n
Ignore
.I \einput
and
.I \einclude
commands.  This allows processing of a file without examining its
subsidiary files.
.\"-----------------------------------------------
.TP
.B \-s
Older versions of
.B detex
would replace control sequences with a space character to prevent
words from running together.  However, this caused accents in the
middle of words to break words, generating `spelling errors' that were
not desirable.  The
.B \-s
option requests the old functionality.
.\"-----------------------------------------------
.TP
.B \-v
Print a version number and date on
.IR stderr ,
and then exit immediately with a success status code (0 on UNIX).
.\"-----------------------------------------------
.TP
.B \-w
Output a word list, one `word' (string of two or more
letters and apostrophes beginning with a letter)
per line, and all other characters ignored.
.IP
Without
.BR \-w ,
the output follows the original, apart with the deletions mentioned
elsewhere.  Newline characters are preserved where possible so that
the lines of output match the input as closely as possible. This helps
relate line-numbered warning and error messages back to the original
source files when other tools are applied to
.BR detex 's
output.
.\"======================================================================
.SH DIAGNOSTICS
Nesting of
.I \einput
commands is allowed but the number of opened files must not exceed the
system's limit on the number of simultaneously opened files.
.PP
.B detex
ignores unrecognized option characters after printing a warning
message.
.\"======================================================================
.SH "ENVIRONMENT VARIABLES"
.TP \w'TEXINPUTS'u+3n
.B TEXINPUTS
\*(Te\& input directory search path. This is a colon-separated list of
directories to search for
.I \einput
and
.I \einclude
files that lack a directory prefix.
.\"======================================================================
.SH FILES
.B detex
requires no additional files beyond those named on its command line.
.\"======================================================================
.SH "SEE ALSO"
.BR dw (1),
.BR emacs (1)
.BR lacheck (1),
.BR tex(1).
.\"======================================================================
.SH BUGS
.B detex
is not a complete
.I \*(Te\&
interpreter, so it can be confused by some constructs.  Most errors
result in too much, rather than too little, output.
.PP
Running \fI\*(La\&\fP source without a
.I \ebegin{document}
through
.B detex
may produce errors.
.PP
Suggestions for improvements are encouraged.
.\"======================================================================
.SH AUTHOR
.nf
Daniel Trinkle
Department of Computer Science
Purdue University
1398 Computer Science Building
West Lafayette, IN 47907-1398
USA
.sp
Email: trinkle@cs.purdue.edu
WWW URL: http://www.cs.purdue.edu/people/trinkle
.fi
